import java.text.NumberFormat;
import java.text.ParseException;

public class PerformanceTester implements Runnable {

  private DateParser dateParser;
  private int numberOfThreads;
  private int numberOfParseCallsPerThread;

  public PerformanceTester(DateParser dateParser, int numberOfThreads, int numberOfParseCallsPerThread) {
    this.dateParser = dateParser;
    this.numberOfThreads = numberOfThreads;
    this.numberOfParseCallsPerThread = numberOfParseCallsPerThread;
  }

  public void run() {
    try {
      for (int i = 0; i < numberOfParseCallsPerThread; i++) {
        dateParser.parse("28/08/2002");
      }
    } catch (ParseException e) {
      e.printStackTrace();
    }
  }

  public double test() {
    Thread[] threads = new Thread[numberOfThreads];
    for (int i = 0; i < threads.length; i++) {
      threads[i] = new Thread(this);
    }
    long startTime = System.currentTimeMillis();
    for (int i = 0; i < threads.length; i++) {
      threads[i].start();
    }
    for (int i = 0; i < threads.length; i++) {
      try {
        threads[i].join();
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
    }
    long endTime = System.currentTimeMillis();
    return (endTime - startTime) / 1000.0;
  }

  public static double[] test(DateParser[] dateParsers, int numberOfThreads,
      int numberOfParseCallsPerThread, int runs) {

    double[] meanDurations = new double[dateParsers.length];
    for (int i = 0; i < dateParsers.length; i++) {
      PerformanceTester tester = new PerformanceTester(dateParsers[i], numberOfThreads, numberOfParseCallsPerThread);
      double totalDuration = 0.0;
      for (int j = 0; j < runs; j++) {
        double duration = tester.test();
        totalDuration += duration;
      }
      meanDurations[i] = totalDuration / runs;
    }
    return meanDurations;
  }

  public static void main(String[] args) {
    if (args.length < 4) {
      printUsage();
      System.exit(1);
    }
    int arg = 0;
    final int maxThreadsExponent = Integer.parseInt(args[arg++]);
    final int totalParseCallsExponent = Integer.parseInt(args[arg++]);
    final int runs = Integer.parseInt(args[arg++]);

    DateParser[] dateParsers = new DateParser[args.length - arg];
    for (int i = 0; i < dateParsers.length; i++) {
      try {
        dateParsers[i] = (DateParser) Class.forName(args[arg++]).newInstance();
      } catch (Exception e) {
        e.printStackTrace();
        System.exit(1);
      }
    }

    double[][] durationMatrix = new double[maxThreadsExponent][dateParsers.length];
    int threads = 1;
    int totalParseCalls = 1 << totalParseCallsExponent;
    for (int i = 0; i < maxThreadsExponent; i++) {
      durationMatrix[i] = test(dateParsers, threads, totalParseCalls / threads, runs);
      threads *= 2;
    }

    threads = 1;
    System.out.print("DateParser implementation,");
    for (int i = 0; i < durationMatrix.length; i++) {
      System.out.print(threads + ",");
      threads *= 2;
    }
    System.out.println();

    NumberFormat format = NumberFormat.getInstance();
    format.setMinimumFractionDigits(2);
    format.setMaximumFractionDigits(2);
    for (int i = 0; i < dateParsers.length; i++) {
      System.out.print(dateParsers[i].getClass().getName() + ",");
      for (int j = 0; j < durationMatrix.length; j++) {
        System.out.print(format.format(durationMatrix[j][i]) + ",");
      }
      System.out.println();
    }
  }

  private static void printUsage() {
      System.err.println("PerformanceTester maxThreadsExponent totalParseCallsExponent runs dateParserClassnames");
  }

}
